import debounce from 'debounce';
import BaseForm from 'controllers/base_form_controller';

export default class extends BaseForm {
  static targets = ['mySelection', 'inputSearch', 'tagsetRadio', 'loader', 'completeness', 'fileInput'];

  connect() {
    super.connect();

    this.addRadioListeners();

    if (this.hasFileInputTarget) {
      this.fileInputTarget.addEventListener('change', this.updateSubmitVisibilityHandler);
    }

    this.inputSearchTarget.addEventListener('keyup', debounce(this.handleKeyup.bind(this), 300));
  }

  disconnect() {
    super.disconnect();

    this.removeRadioListeners();

    if (this.hasFileInputTarget) {
      this.fileInputTarget.removeEventListener('change', this.updateSubmitVisibilityHandler);
    }

    this.inputSearchTarget.removeEventListener('keyup', this.handleKeyup.bind(this));
  }

  addRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.addEventListener('change', this.updateSubmitVisibilityHandler);
    });
  }

  removeRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.removeEventListener('change', this.updateSubmitVisibilityHandler);
    });
  }

  calculateShouldShowSubmit() {
    const allFieldsFilled = Array.from(this.requiredFields).every(el => this.isNotEmptyField(el));
    const isAddActionValid = this.addActionTarget.checked ? !!this.nextActionTarget.value : true;
    const fileOrRadioRequired = this.hasFileInputTarget ? this.fileInputTarget.files.length > 0 : this.tagsetRadioTargets.some(radio => radio.checked);
    return allFieldsFilled && isAddActionValid && fileOrRadioRequired;
  }

  handleKeyup(event) {
    const query = event.target.value;

    if (query.length > 2) {
      this.search(query);
    }
  }

  search(query) {
    const url = `/spirit_hub/tagsets/search_directories?query=${encodeURIComponent(query)}`;
    this.loaderTarget.classList.remove('hidden');

    fetch(url, { headers: { Accept: 'text/vnd.turbo-stream.html' } })
      .then(response => response.text())
      .then(html => {
        this.removeRadioListeners();
        Turbo.renderStreamMessage(html);
        requestAnimationFrame(() => {
          this.addRadioListeners();
          this.updateSubmitVisibility.bind(this);
          this.loaderTarget.classList.add('hidden');
        });
      })
      .catch(error => console.error('Error:', error));
  }

  tagsetChanged(event) {
    const selectedValue = event.target.value;
    const isChecked = event.target.checked;
    const matchingDivs = this.element.querySelectorAll(`div[data-tagset-id="${selectedValue}"]`);

    this.completenessTargets.forEach(div => div.classList.add('hidden'));

    if (isChecked) {
      matchingDivs.forEach(div => div.classList.remove('hidden'));
    }
  }
}
