# frozen_string_literal: true

module SpiritHub
  class PhasesController < ApplicationController
    def inline_edit
      skip_authorization
      skip_policy_scope

      key = params[:key].to_sym
      model = SpiritHub::Phase.find(params[:id])

      render turbo_stream: turbo_stream.update("inline_edit_#{key}_#{model.id}", partial: 'spirit_hub/phases/inline_edit', locals: { model:, key: })
    end

    def update_inline
      skip_authorization
      skip_policy_scope

      model = SpiritHub::Phase.find(params[:id])
      key = inline_update_params[:key].to_sym

      model.update(key => inline_update_params[key])
      value = find_inline_value(model, key)

      flash.now[:notice] = 'Valeur mise à jour'
      update_flash_message

      @turbo_actions << turbo_stream.update("inline_value_#{key}_#{model.id}", partial: 'spirit_hub/common/inline_value', locals: { model:, value:, key: })

      render turbo_stream: @turbo_actions
    end

    private

    def inline_update_params
      params[:phase].permit(:key, :initial_date, :postponed_date, :actual_date)
    end

    def find_inline_value(model, key)
      model.send(key)&.strftime('%d/%m/%Y') || 'N/A'
    end
  end
end
