# frozen_string_literal: true

module SpiritHub
  module TeamActivities
    class CollaboratorController < ApplicationController
      before_action :set_collaborator
      before_action :find_models, only: %i[index sort]

      include Datatablable
      include Sectionable

      PER_PAGE = 10

      def index
        skip_authorization
        skip_policy_scope

        respond_to(&:html)
      end

      def sort
        skip_authorization
        skip_policy_scope

        render partial: 'spirit_hub/team_activities/collaborator/table'
      end

      private

      def table_partial
        'spirit_hub/team_activities/collaborator/table'
      end

      def set_collaborator
        @collaborator = User.find(params[:collaborator_id])
      end

      def default_columns
        {
          type: { label: 'Type', sortable: true },
          cleaned_main_key: { label: 'Libellé', sortable: true },
          doc_file_size: { label: 'Poids', sortable: true },
          updated_at: { label: 'Dernière modification', sortable: true },
          last_current_user_name: { label: 'Provenance', sortable: true },
          informed: { label: 'Partagé', sortable: false },
          current_state: { label: 'Statut', sortable: true }
        }.freeze
      end

      def find_models
        @all_to_be_signed = @collaborator.to_be_signed_docs
        @all_to_complete = @collaborator.to_complete_docs
        @all_to_validate = @collaborator.to_validate_docs
        @all_responses = @collaborator.responses_docs
        setup_definitions(%i[to_be_signed to_complete to_validate responses])

        set_user_favorites
      end
    end
  end
end
