# frozen_string_literal: true

Log.class_eval do
  CREATE_CODES = [
    'DOC_CREATED',
    'DOC_CREATED_FROM_TEMPLATE',
    'DOC_ADDED_TO_COMPLETNESS',
    'OPERATION_CREATED',
    'DIRECTORY_CREATED'
  ].freeze

  DELETE_CODES = [
    'DOC_DELETED'
  ].freeze

  SHARE_CODES = [
    'DOC_SHARED',
    'MULTIPLE_DOC_SHARED',
    'SHARE_LINK_CREATED',
    'COMMENT_ADDED_TO_LINK'
  ].freeze

  CLASSIFY_CODES = [
    'DOC_CLASSIFIED'
  ].freeze

  EDIT_CODES = [
    'DOC_EDITED_IN_OFFICE_ONLINE',
    'DIRECTORY_INFORMATION_UPDATED'
  ].freeze

  PROCESS_CODES = [
    'DOC_ASSIGNED',
    'DOC_SIGNED',
    'DOC_TO_SIGN',
    'DOC_ANSWERED'
  ].freeze

  SENT_TO_CODES = %w[
    DOC_VALIDATED
    DOC_REFUSED
    DOC_SENT_TO_SHARE
    DOC_SENT_TO_VALIDATE
    DOC_SENT_TO_COMPLETE
    DOC_SENT_TO_ANSWER
    DOC_SENT_TO_SIGN
  ].freeze

  class << self
    %i[create delete share classify edit sent_to].each do |log_type|
      define_method("#{log_type}_action_log") do |code, opts = {}|
        raise "#{log_type} action code missing" if code.nil?
        raise "#{log_type} action code invalid" unless "#{log_type.upcase}_CODES".constantize.include?(code)

        action_log(code, opts)
      end
    end

    private

    def action_log(code, opts)
      tagset = Tagset.find_by(id: opts[:tagset_id])
      user = User.find_by(id: opts[:user_id])
      recipients = find_recipients_names(opts[:recipients])

      message_options = {
        title: opts[:title].presence || tagset&.title,
        username: user&.full_name || user&.email,
        recipients:,
        link: opts[:link],
        recipient: opts[:recipient],
        directory: opts[:directory]
      }

      action_msg = [I18n.t(code, scope: 'log.action', **message_options)]
      action_msg << opts[:action_msg].presence
      Log.perform_later(
        code:,
        data: 'hubdoc',
        tagset_id: tagset&.id || opts[:tagset_id],
        user_id: user&.id || opts[:user_id],
        action: action_msg.compact.join(' : ')
      )
    end

    def find_recipients_names(recipients)
      formated_recipients = recipients.to_a.map do |id|
        if id.to_s.include?('g_')
          "#{Group.find(id.to_s.gsub('g_', ''))&.name} (Groupe)"
        elsif id.to_s.include?('c_')
          "#{Doc.find(id.to_s.gsub('c_', ''))&.email} (Contact)"
        else
          User.find_by(id:)&.full_name
        end
      end

      formated_recipients.compact&.to_sentence(last_word_connector: ' et ')
    end
  end
end
