# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Log, type: :model) do
  describe 'find_recipients_names' do
    let!(:recipients) { [] }

    subject { Log.send(:find_recipients_names, recipients) }

    it 'empty recipients' do
      expect(subject).to eq('')
    end

    context 'with users recipients' do
      let!(:johndoe) { create(:user, first_name: 'John', last_name: 'Doe') }
      let!(:janedoe) { create(:user, first_name: 'Jane', last_name: 'Doe') }
      let!(:recipients) { [johndoe.id.to_s, janedoe.id.to_s] }

      it 'returns the full name of the users' do
        expect(subject).to eq('John Doe et Jane Doe')
      end
    end

    context 'with group recipients' do
      let!(:groupe1) { create(:group, name: 'Groupe 1') }
      let!(:recipients) { ["g_#{groupe1.id}"] }

      it 'returns the name of the group' do
        expect(subject).to eq('Groupe 1 (Groupe)')
      end
    end

    context 'with contact recipients' do
      let!(:contact1) { create(:contact, superfields: { 'email_address_1' => 'test@mail.com' }) }
      let!(:recipients) { ["c_#{contact1.id}"] }

      it 'returns the email of the contact' do
        expect(subject).to eq('test@mail.com (Contact)')
      end
    end

    context 'with mixed recipients' do
      let!(:johndoe) { create(:user, first_name: 'John', last_name: 'Doe') }
      let!(:janedoe) { create(:user, first_name: 'Jane', last_name: 'Doe') }
      let!(:groupe1) { create(:group, name: 'Groupe 1') }

      let!(:recipients) { [johndoe.id.to_s, janedoe.id.to_s, "g_#{groupe1.id}"] }

      it 'returns the full name of the users and the name of the group' do
        expect(subject).to eq('John Doe, Jane Doe et Groupe 1 (Groupe)')
      end
    end
  end
end
